-----------------------------------------------------------------------------------------
-- Generic functions to add to library
-----------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------
-- Initialize global faction lists
-----------------------------------------------------------------------------------------
function CF_InitFactions(activity)
	print ("CF_InitFactions");
   	CF_CPUTeam = Activity.TEAM_2;
	CF_PlayerTeam = Activity.TEAM_1;
	CF_RogueTeam = -1;
	CF_MOIDLimit = 210;
	
	CF_MaxCPUPlayers = 3
	CF_MaxSaveGames = 6
	CF_MaxItemsPerPreset = 3
	CF_MaxAssaultPresets = 6
	CF_MaxDefenders = 8
	CF_MaxUnitsPerDropship = 3
	CF_TickInterval = 500
	CF_AttackEventExpire = 2
	CF_MaxAttackEventSlots = 20
	CF_MessageInterval = 3500
	CF_AllyMissionProbability = 0.015
	CF_AITerrAttackCooldown = 15

	CF_AllyItemProbability = 0.15;--0.10
	-- Time in tactical ticks to show ally reward message
	CF_AllyRewardShowInterval = 3;
	
	CF_DisableAllyMissions = true
	
	-- If this is true UL2 will use the new diplomacy model where player can order ally
	-- reinforcements using ally points
	CF_UseNewDiplomacyModel	= true
	CF_AllyReinforcementsBasePrice = 500
	
	CF_LaunchActivities = true
	CF_MissionReturnInterval = 5000

	CF_TacticalTickInterval = 1000
	CF_TacticalHackTime = 30
	CF_StealAmount = 10
	CF_ActiveConsolesRatio = 0.8

	-- To make HQ assaults harder we'll boost AI defenders with more money
	-- If player didn't destroy facilities first, than AI will get more money for defence
	-- Total money will be Facilities * Levels * CF_BasicAIGoldBonusPerFacility * AI income bonus
	CF_BasicAIGoldBonusPerFacilityLevel = 100

	-- How much percents of price to add if player and ally factions natures are not the same
	CF_SynthetsToOrganicRatio = 0.70
	
	CF_FogOfWarEnabled = false; --true -- Gameplay value
	CF_FogOfWarResolution = 100
	
	CF_SendTroopsToBrain = true;--true -- Gameplay value
	
	CF_DebugSuperWeapons = false;--false -- Gameplay value
	CF_DebugEnableRandomActivity = false;--false -- Gameplay value
	
	CF_AIDebugOutput = false;--false -- Gameplay value
	CF_AIResourcesOutput = false;--false -- Gameplay value
	
	-- Auto save data to Current.dat every N ticks
	CF_AutoSaveInterval = 25
	
	-- Increase tick time when debugging to make it simpler to track the console messages
	if CF_AIDebugOutput then
		CF_TickInterval = CF_TickInterval * 2
	end
	
	
	CF_Factions = {};
	
	CF_Nobody = "Nobody";
	CF_PlayerFaction = "Nobody";
	CF_CPUFaction = "Nobody";
	
	CF_MissionEndTimer = Timer();
	CF_StartReturnCountdown = false;
	CF_Activity = activity;
	
	CF_FactionIds = {};
	CF_FactionNames = {};
	CF_FactionDescriptions = {};
	CF_FactionPlayable = {};

	CF_ScanBonuses = {}
	CF_RelationsBonuses = {}
	CF_ExpansionBonuses = {}

	CF_MineBonuses = {}
	CF_LabBonuses = {}
	CF_AirfieldBonuses = {}
	CF_SuperWeaponBonuses = {}
	CF_FactoryBonuses = {}
	CF_CloneBonuses = {}
	CF_HospitalBonuses = {}

	CF_HackTimeBonuses = {}
	CF_HackRewardBonuses = {}
	
	CF_DropShipCapacityBonuses = {}

	-- Special arrays for factions with pre-equipped items
	-- Everything in this array (indexed by preset name) will not be included by inventory saving routines
	CF_DiscardableItems = {}
	-- Everything in this array will be marked for deletion after actor is created
	CF_ItemsToRemove = {}
	
	CF_BrainHuntRatios = {}
	
	CF_PreferedBrainInventory = {}
	
	CF_SuperWeaponScripts = {}

	CF_ResearchQueues = {}

	-- Specify presets which are not affected by tactical AI unit management
	CF_UnassignableUnits = {}
	
	-- Set this to true if your faction uses pre-equipped actors
	CF_PreEquippedActors = {}

	CF_PresetNames = {"Infantry 1", "Infantry 2", "Sniper", "Shotgun", "Heavy 1", "Heavy 2", "Armor 1", "Armor 2", "Engineer", "Defender"}
	CF_PresetTypes = {INFANTRY1 = 1, INFANTRY2 = 2, SNIPER = 3, SHOTGUN = 4, HEAVY1 = 5, HEAVY2 = 6, ARMOR1 = 7, ARMOR2 = 8, ENGINEER = 9, DEFENDER = 10}
	
	-- Arrays with a combination of presets used by this faction, script will randomly select presets for deployment from this arrays if available
	CF_PreferedTacticalPresets = {}
	
	-- Default presets array, everything is evenly selected by AI
	CF_DefaultTacticalPresets = {	
			CF_PresetTypes.INFANTRY1,
			CF_PresetTypes.INFANTRY2,
			CF_PresetTypes.SNIPER, 
			CF_PresetTypes.SHOTGUN, 
			CF_PresetTypes.HEAVY1, 
			CF_PresetTypes.HEAVY2, 
			CF_PresetTypes.ARMOR1, 
			CF_PresetTypes.ARMOR2, 
			CF_PresetTypes.ENGINEER	
		}	
	
	CF_FactionAIModels = {}
	
	CF_WeaponTypes = {PISTOL = 0, RIFLE = 1, SHOTGUN = 2, SNIPER = 3, HEAVY = 4, SHIELD = 5, DIGGER = 6, GRENADE = 7, TOOL = 8}
	CF_ActorTypes = {LIGHT = 0, HEAVY = 1, ARMOR = 2, TURRET = 3}
	CF_FactionTypes = {ORGANIC = 0, SYNTHETIC = 1}

	CF_ItmNames = {}
	CF_ItmPresets = {}
	CF_ItmModules = {}
	CF_ItmPrices = {}
	CF_ItmDescriptions = {}
	CF_ItmUnlockData = {}
	CF_ItmClasses = {}
	CF_ItmTypes = {}
	CF_ItmPowers = {} -- AI will select weapons based on this value

	CF_ActNames = {}
	CF_ActPresets = {}
	CF_ActModules = {}
	CF_ActPrices = {}
	CF_ActDescriptions = {}
	CF_ActUnlockData = {}
	CF_ActClasses = {}
	CF_ActTypes = {}
	CF_EquipmentTypes = {}		
	CF_ActPowers = {}
	CF_ActOffsets = {}
	
	CF_RequiredModules	 = {}

	CF_FactionNatures = {}
	
	CF_Brains = {};
	CF_BrainModules = {};
	CF_BrainClasses = {};
	CF_BrainPrices = {}
	
	CF_Crafts = {};
	CF_CraftModules = {};
	CF_CraftClasses = {};
	CF_CraftPrices = {}

	local factionstorage = "./Warfare2.rte/Factions/"
	
	-- Load factions
	if CF_IsFilePathExists("./Factions2/Factions.cfg") then
		CF_FactionFiles = CF_ReadFactionsList("Factions2/Factions.cfg")
		factionstorage = "./Factions2/"
		print ("USING FACTIONS2 FOLDER!")
	else
		CF_FactionFiles = CF_ReadFactionsList("Warfare2.rte/Factions/Factions.cfg")
	end
	
	--print (#CF_FactionFiles)

	-- Load factions data
	for i = 1, #CF_FactionFiles do
		--print("Loading "..CF_FactionFiles[i])
		f = loadfile(factionstorage..CF_FactionFiles[i])
		if f ~= nil then
			f()
		
			local id  = CF_Factions[#CF_Factions]
		
			-- Check if faction modules installed. Check only works with old v1 or most new v2 faction files.
			--print(CF_InfantryModules[CF_Factions[#CF_Factions]])
			for m = 1, #CF_RequiredModules[id] do
				local module = CF_RequiredModules[id][m];
			
				if module ~= nil then
					if PresetMan:GetModuleID(module) == -1 then
						CF_FactionPlayable[id] = false;
						print ("ERROR!!! "..id.." DISABLED!!! "..CF_RequiredModules[id][m].." NOT FOUND!!!")
					end
				end
			end
			
			-- Assume that faction file is correct
			local factionok = true;
			local err = "";
			
			-- Verify faction file data and add mission values if any
			-- Verify items
			for i = 1, #CF_ItmNames[id] do
				if CF_ItmModules[id][i] == nil then
					factionok = false;
					err = "CF_ItmModules is missing."
				end

				if CF_ItmPrices[id][i] == nil then
					factionok = false;
					err = "CF_ItmPrices is missing."
				end

				if CF_ItmDescriptions[id][i] == nil then
					factionok = false;
					err = "CF_ItmDescriptions is missing."
				end
				
				if CF_ItmUnlockData[id][i] == nil then
					factionok = false;
					err = "CF_ItmUnlockData is missing."
				end

				if CF_ItmTypes[id][i] == nil then
					factionok = false;
					err = "CF_ItmTypes is missing."
				end

				if CF_ItmPowers[id][i] == nil then
					factionok = false;
					err = "CF_ItmPowers is missing."
				end
				
				-- If something is wrong then disable faction and print error message
				if not factionok then
					CF_FactionPlayable[id] = false;
					print ("ERROR!!! "..id.." DISABLED!!! "..CF_ItmNames[id][i].." : "..err)
					break;
				end
			end

			-- Assume that faction file is correct
			local info = {}
			local data = {}
			
			-- Verify faction generic data
			info[#info + 1] = "CF_FactionNames"
			data[#info] = CF_FactionNames[id]

			info[#info + 1] = "CF_FactionDescriptions"
			data[#info] = CF_FactionDescriptions[id]

			info[#info + 1] = "CF_FactionPlayable"
			data[#info] = CF_FactionPlayable[id]

			info[#info + 1] = "CF_RequiredModules"
			data[#info] = CF_RequiredModules[id]

			info[#info + 1] = "CF_FactionNatures"
			data[#info] = CF_FactionNatures[id]

			info[#info + 1] = "CF_ScanBonuses"
			data[#info] = CF_ScanBonuses[id]
			
			info[#info + 1] = "CF_RelationsBonuses"
			data[#info] = CF_RelationsBonuses[id]

			info[#info + 1] = "CF_ExpansionBonuses"
			data[#info] = CF_ExpansionBonuses[id]

			info[#info + 1] = "CF_MineBonuses"
			data[#info] = CF_MineBonuses[id]

			info[#info + 1] = "CF_LabBonuses"
			data[#info] = CF_LabBonuses[id]

			info[#info + 1] = "CF_AirfieldBonuses"
			data[#info] = CF_AirfieldBonuses[id]

			info[#info + 1] = "CF_SuperWeaponBonuses"
			data[#info] = CF_SuperWeaponBonuses[id]

			info[#info + 1] = "CF_FactoryBonuses"
			data[#info] = CF_FactoryBonuses[id]

			info[#info + 1] = "CF_CloneBonuses"
			data[#info] = CF_CloneBonuses[id]

			info[#info + 1] = "CF_HospitalBonuses"
			data[#info] = CF_HospitalBonuses[id]

			info[#info + 1] = "CF_Brains"
			data[#info] = CF_Brains[id]

			info[#info + 1] = "CF_BrainModules"
			data[#info] = CF_BrainModules[id]

			info[#info + 1] = "CF_BrainClasses"
			data[#info] = CF_BrainClasses[id]

			info[#info + 1] = "CF_BrainPrices"
			data[#info] = CF_BrainPrices[id]

			info[#info + 1] = "CF_Crafts"
			data[#info] = CF_Crafts[id]

			info[#info + 1] = "CF_CraftModules"
			data[#info] = CF_CraftModules[id]

			info[#info + 1] = "CF_CraftClasses"
			data[#info] = CF_CraftClasses[id]

			info[#info + 1] = "CF_CraftPrices"
			data[#info] = CF_CraftPrices[id]
			
			for i = 1, #info do
				if data[i] == nil then
					CF_FactionPlayable[id] = false;
					print ("ERROR!!! "..id.." DISABLED!!! "..info[i].." is missing")
					break;
				end
			end
			
			-- Assume that faction file is correct
			local factionok = true;
			local err = "";
			
			-- Verify actors
			for i = 1, #CF_ActNames[id] do
				if CF_ActModules[id][i] == nil then
					factionok = false;
					err = "CF_ActModules is missing."
				end

				if CF_ActPrices[id][i] == nil then
					factionok = false;
					err = "CF_ActPrices is missing."
				end

				if CF_ActDescriptions[id][i] == nil then
					factionok = false;
					err = "CF_ActDescriptions is missing."
				end
				
				if CF_ActUnlockData[id][i] == nil then
					factionok = false;
					err = "CF_ActUnlockData is missing."
				end

				if CF_ActTypes[id][i] == nil then
					factionok = false;
					err = "CF_ActTypes is missing."
				end

				if CF_ActPowers[id][i] == nil then
					factionok = false;
					err = "CF_ActPowers is missing."
				end
				
				-- If something is wrong then disable faction and print error message
				if not factionok then
					CF_FactionPlayable[id] = false;
					print ("ERROR!!! "..id.." DISABLED!!! "..CF_ActNames[id][i].." : "..err)
					break;
				end
			end
		else
			print ("ERROR!!! Could not load: "..CF_FactionFiles[i])
		end
	end
	
	--print("Factions loaded:")
	for i = 1 , #CF_Factions do
		--print(CF_Factions[i])
	end	
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function CF_MakeDropship(team , pos , faction, ignoremoids)
	-- We need to ignore moids when we're spawning brain drop ship
	if not ignoremoids then
		if MovableMan:GetMOIDCount() > CF_MOIDLimit then
			print ("Create dropship: MOID limit reached")
			return nil
		end
	end

	local preset = CF_Crafts[faction]
	local class = CF_CraftClasses[faction]
	local module = CF_CraftModules[faction]
	
	local ship
	
	if class == nil then
		class = "AHuman"
	end
	
	if class == "ACDropShip" then
		ship = CreateACDropShip(preset, module)
	elseif class == "ACRocket" then
		ship = CreateACRocket(preset, module)
	end	
	
	if ship then
		ship.Team = team
		if pos then
			ship.Pos = pos
		end
		ship.AIMode = Actor.AIMODE_DELIVER
		ship:SetControllerMode(Controller.CIM_AI, -1);
	end
	
	return ship
end
-----------------------------------------------------------------------------------------
-- f - faction, returns brain actor, price
-----------------------------------------------------------------------------------------
function CF_MakeBrain(f)
	local actor = nil
	local weapon = nil;
	local price = 0
	
	actor = CF_MakeActorNoMatterWhat(CF_Brains[f], CF_BrainClasses[f], CF_BrainModules[f])
	
	if actor ~= nil then
		price = CF_BrainPrices[f];
		actor.AIMode = Actor.AIMODE_SENTRY;
	end
	
	return actor, price;
end
-----------------------------------------------------------------------------------------
-- f - faction, weapontype - some CF_WeaponTypes value, returns list with indexes of weapons 
-- of desired type or empty list of there's nothing
-----------------------------------------------------------------------------------------
function CF_MakeListOfWeapons(f, weapontype)
	local list = {}

	for i = 1, #CF_ItmNames[f] do
		if CF_ItmTypes[f][i] == weapontype then
			list[#list + 1] = i
		end
	end
	
	return list
end
-----------------------------------------------------------------------------------------
-- f - faction, actortype - some CF_ActorTypes value, returns list with indexes of actors 
-- of desired type or empty list of there's nothing
-----------------------------------------------------------------------------------------
function CF_MakeListOfActors(f, actortype)
	local list = {}

	for i = 1, #CF_ActNames[f] do
		if CF_ActTypes[f][i] == actortype then
			list[#list + 1] = i
		end
	end
	
	return list
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function CF_GetAnyOfThoseWeapons(f, wt, aiusable)
	local weap = nil;

	for i = 1, #wt do
		local list = CF_MakeListOfWeapons(f, wt[i])
		
		if #list > 0 then
			weap = list[math.random(#list)]
			return weap
		end
	end
	
	return nil;
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function CF_GetRandomWeaponIndex(f, wt, aiusableonly)
	local ok = false;
	local weap = nil;
	local emergencybreak = 0
	
	while (not ok) do
		r = math.random(#wt)
		
		local list = CF_MakeListOfWeapons(f, wt[r])
		
		if #list > 0 then
			weap = list[math.random(#list)]
			
			if aiusableonly then
				if CF_ItmPowers[f][weap] > 0 then
					ok = true
				end
			else
				ok = true
			end
		end
		
		emergencybreak = emergencybreak + 1
		
		if emergencybreak == 40 then
			weap = nil
			break;
		end
	end
	
	return weap;
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function CF_GetRandomActorIndex(f, wt, aiusableonly)
	local ok = false;
	local act = nil;
	local emergencybreak = 0
	
	while (not ok) do
		r = math.random(#wt)
		
		local list = CF_MakeListOfActors(f, wt[r])
		
		if #list > 0 then
			act = list[math.random(#list)]
			
			if aiusableonly then
				if CF_ActPowers[f][act] > 0 then
					ok = true
				end
			else
				ok = true
			end
		end
		
		emergencybreak = emergencybreak + 1
		
		if emergencybreak == 40 then
			act = nil
			break;
		end
	end
	
	return act;
end

-----------------------------------------------------------------------------------------
-- Replace default dummy actors placed manually to faction-specific actors
-----------------------------------------------------------------------------------------
function CF_ReplaceActors(collection)
	print ("CF_ReplaceActors");
	local team;
	local toremove;
	local lpos;
	local viewpoint;
	local colls = {};
    -- Set up AI modes for the Actors that have been added to the scene by the scene definition
    for actor in collection do
		toremove = false; 
		
		if actor.Team ~= CF_RogueTeam then
			--print(actor.PresetName);
			-- Replace soldier units
			if actor.ClassName == "AHuman" or actor.ClassName == "ACrab" then
				lpos = Vector(actor.Pos.X , actor.Pos.Y);
				actor.Pos = Vector(0 , 30000);
				toremove = true;
				if actor.Team == CF_PlayerTeam then
					CF_SpawnRandomInfantry(CF_PlayerTeam , lpos , CF_PlayerFaction , Actor.AIMODE_SENTRY);
				else
					CF_SpawnRandomInfantry(CF_CPUTeam , lpos , CF_CPUFaction , Actor.AIMODE_SENTRY);
				end
			end
		end
		
		if toremove then 
			actor.Lifetime = 0
			ActivityMan:GetActivity():ReportDeath(actor.Team , -1)
		end
    end
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function CF_SpawnRandomInfantry(team , pos , faction , aimode)
	--print ("CF_SpawnRandomInfantry");
	local actor = nil;
	local r1, r2;
	local item;
	
	if MovableMan:GetMOIDCount() < CF_MOIDLimit then
		-- Find AHuman
		local ok = false;
		-- Emergency counter in case we don't have AHumans in factions
		local counter = 0
		
		while (not ok) do
			ok = false;
			r1 = math.random(#CF_ActNames[faction])

			if (CF_ActClasses[faction][r1] == nil or CF_ActClasses[faction][r1] == "AHuman") and CF_ActTypes[faction][r1] ~= CF_ActorTypes.ARMOR then
				ok = true;
			end
			
			-- Break to avoid endless loop
			counter = counter + 1
			if counter > 20 then
				break
			end
		end
	
		actor = CF_MakeActor(CF_ActPresets[faction][r1] , CF_ActClasses[faction][r1], CF_ActModules[faction][r1]);
		
		if actor ~= nil then
			-- Check if this is pre-equipped faction
			local preequipped = false
			
			if CF_PreEquippedActors[faction] ~= nil and CF_PreEquippedActors[faction] then
				preequpped = true
			end			
		
			if not preequipped then
				-- Find rifle
				local ok = false;
				-- Emergency counter in case we don't have AHumans in factions
				local counter = 0
				
				while (not ok) do
					ok = false;
					r2 = math.random(#CF_ItmNames[faction])
			
					if CF_ItmTypes[faction][r2] == CF_WeaponTypes.RIFLE or CF_ItmTypes[faction][r2] == CF_WeaponTypes.SHOTGUN or CF_ItmTypes[faction][r2] == CF_WeaponTypes.SNIPER then
						ok = true;
					end
					
					-- Break to avoid endless loop
					counter = counter + 1
					if counter > 40 then
						break
					end
				end
			
				item = CF_MakeItem(CF_ItmPresets[faction][r2] , CF_ItmClasses[faction][r2], CF_ItmModules[faction][r2]);
				
				if item ~= nil then
					actor:AddInventoryItem(item);
				end
			end
			
			actor.AIMode = aimode;
			actor.Team = team;
			
			if pos ~= nil then
				actor.Pos = pos;
				MovableMan:AddActor(actor);
			else
				return actor
			end
		end
	end
	
	return nil
end
-----------------------------------------------------------------------------------------
-- For a given char returns its index, width, vector offsset  if any
-----------------------------------------------------------------------------------------
function CF_GetCharData(char)
	if CF_Chars == nil then
		CF_Chars = {}
		CF_Chars[" "] = {1,8,nil};
		CF_Chars["!"] = {2,5,Vector(-3,0)};
		CF_Chars["\""] = {3,8,Vector(0,-2)};
		CF_Chars["#"] = {4,8,nil};
		CF_Chars["$"] = {5,8,nil};
		CF_Chars["%"] = {6,8,nil};
		CF_Chars["&"] = {7,6,Vector(-2,0)};
		CF_Chars["`"] = {8,5,Vector(-3,-2)};
		CF_Chars["("] = {9,6,Vector(-1,0)};
		CF_Chars[")"] = {10,6,Vector(-2,0)};
		CF_Chars["*"] = {11,6,Vector(-2,0)};
		CF_Chars["+"] = {12,8,nil};
		CF_Chars[","] = {13,5,Vector(-3,5)};
		CF_Chars["-"] = {14,9,nil};
		CF_Chars["."] = {15,5,Vector(-3,4)};
		CF_Chars["/"] = {16,6,Vector(-1,0)};
		CF_Chars["0"] = {17,8,nil};
		CF_Chars["1"] = {18,6,Vector(-2,0)};
		CF_Chars["2"] = {19,8,nil};
		CF_Chars["3"] = {20,8,nil};
		CF_Chars["4"] = {21,8,nil};
		CF_Chars["5"] = {22,8,nil};
		CF_Chars["6"] = {23,8,nil};
		CF_Chars["7"] = {24,8,nil};
		CF_Chars["8"] = {25,8,nil};
		CF_Chars["9"] = {26,8,nil};
		CF_Chars[":"] = {27,5,Vector(-3,-1)};
		CF_Chars[";"] = {28,5,Vector(-3,-1)};
		CF_Chars["<"] = {29,7,Vector(-1,0)};
		CF_Chars["="] = {30,8,Vector(0,-1)};
		CF_Chars[">"] = {31,7,Vector(-1,0)};
		CF_Chars["?"] = {32,8,nil};
		CF_Chars["@"] = {33,11,Vector(0,-1)};
		CF_Chars["A"] = {34,8,nil};
		CF_Chars["B"] = {35,8,nil};
		CF_Chars["C"] = {36,8,Vector(0,-3)};
		CF_Chars["D"] = {37,8,nil};
		CF_Chars["E"] = {38,8,nil};
		CF_Chars["F"] = {39,8,nil};
		CF_Chars["G"] = {40,8,nil};
		CF_Chars["H"] = {41,8,nil};
		CF_Chars["I"] = {42,5,Vector(-2,0)};
		CF_Chars["J"] = {43,8,nil};
		CF_Chars["K"] = {44,8,nil};
		CF_Chars["L"] = {45,8,Vector(0,3)};
		CF_Chars["M"] = {46,10,Vector(1,-1)};
		CF_Chars["N"] = {47,8,nil};
		CF_Chars["O"] = {48,8,nil};
		CF_Chars["P"] = {49,8,nil};
		CF_Chars["Q"] = {50,8,nil};
		CF_Chars["R"] = {51,8,nil};
		CF_Chars["S"] = {52,8,nil};
		CF_Chars["T"] = {53,7,Vector(-1,0)};
		CF_Chars["U"] = {54,8,nil};
		CF_Chars["V"] = {55,8,nil};
		CF_Chars["W"] = {56,10,Vector(2,0)};
		CF_Chars["X"] = {57,8,nil};
		CF_Chars["Y"] = {58,8,nil};
		CF_Chars["Z"] = {59,8,nil};
		CF_Chars["["] = {60,6,Vector(-1,0)};
		CF_Chars["\\"] = {61,6,Vector(-1,0)};
		CF_Chars["]"] = {62,6,Vector(-1,0)};
		CF_Chars["^"] = {63,8,Vector(-1,-3)};
		CF_Chars["_"] = {64,8,Vector(0,4)};
		CF_Chars["'"] = {65,8,Vector(0,-3)};
		CF_Chars["a"] = {66,8,nil};
		CF_Chars["b"] = {67,8,nil};
		CF_Chars["c"] = {68,8,nil};
		CF_Chars["d"] = {69,8,nil};
		CF_Chars["e"] = {70,8,nil};
		CF_Chars["f"] = {71,7,Vector(-1,0)};
		CF_Chars["g"] = {72,8,nil};
		CF_Chars["h"] = {73,8,nil};
		CF_Chars["i"] = {74,5,Vector(-3,0)};
		CF_Chars["j"] = {75,6,Vector(-2,0)};
		CF_Chars["k"] = {76,8,nil};
		CF_Chars["l"] = {77,5,Vector(-3,0)};
		CF_Chars["m"] = {78,10,nil};
		CF_Chars["n"] = {79,8,nil};
		CF_Chars["o"] = {80,8,nil};
		CF_Chars["p"] = {81,8,nil};
		CF_Chars["q"] = {82,8,nil};
		CF_Chars["r"] = {83,8,nil};
		CF_Chars["s"] = {84,8,nil};
		CF_Chars["t"] = {85,8,nil};
		CF_Chars["u"] = {86,8,nil};
		CF_Chars["v"] = {87,8,nil};
		CF_Chars["w"] = {88,10,Vector(1,0)};
		CF_Chars["x"] = {89,8,nil};
		CF_Chars["y"] = {90,8,nil};
		CF_Chars["z"] = {91,8,nil};
		CF_Chars["{"] = {92,7,nil};
		CF_Chars["|"] = {93,7,nil};
		CF_Chars["}"] = {94,7,nil};
		CF_Chars["~"] = {95,8,nil};
	end
	
	local i = nil;
	
	i = CF_Chars[char];
	
	if i == nil then
		i = {96,8,nil};
	end

	return i[1], i[2] - 2, i[3];
end
--------------------------------------------------------------------------
-- Return size of string in pixels
-----------------------------------------------------------------------------
function CF_GetStringPixelWidth(str)
	local len = 0;
	for i = 1, #str do
		local cindex, cwidth, coffset = CF_GetCharData(string.sub(str , i , i))
		len = len + cwidth;
	end
	return len;
end
-----------------------------------------------------------------------------
--
-----------------------------------------------------------------------------
function CF_Split(str, pat)
   local t = {}  -- NOTE: use {n = 0} in Lua-5.0
   local fpat = "(.-)" .. pat
   local last_end = 1
   local s, e, cap = str:find(fpat, 1)
   while s do
      if s ~= 1 or cap ~= "" then
	 table.insert(t , cap)
      end
      last_end = e+1
      s, e, cap = str:find(fpat, last_end)
   end
   if last_end <= #str then
      cap = str:sub(last_end)
      table.insert(t, cap)
   end
   return t
end
-----------------------------------------------------------------------------
-- Draw string on screen at speicified pos not wider that width and not higher than height
-----------------------------------------------------------------------------
function CF_DrawString(str , pos , width , height)
	local x;
	local y;
	local chr;
	
	x = pos.X
	y = pos.Y
	
	local words = CF_Split(str," ");
	for w = 1, #words do
		words[w] = words[w];
		
		if x + CF_GetStringPixelWidth(words[w]) > pos.X + width then
			x = pos.X
			y = y + 11
			
			if y > pos.Y + height then
				return
			end
		end
	
		for i = 1, #words[w] do
			chr = string.sub(words[w] , i , i);
			
			if chr == "\n" then
				x = pos.X
				y = y + 11
				
				if y > pos.Y + height then
					return
				end
			else
				local cindex, cwidth, coffset = CF_GetCharData(chr)
				
				local pix = CreateMOPixel("Ltr"..cindex);
				local offset = coffset
				if offset ~= nil then
					pix.Pos = Vector(x , y) + offset;
				else
					pix.Pos = Vector(x , y);
				end
				
				MovableMan:AddParticle(pix);
				
				x = x + cwidth
			end
		end
		
		-- Simulate space character, only if we're not at the begining of new line
		if x ~= pos.X then
			x = x + 6
		end
	end
end
-----------------------------------------------------------------------------
-- Converts time in second to string h:mm:ss
-----------------------------------------------------------------------------
function CF_ConvertTimeToString(timenum)
	local timestr = "";

	local hours = (timenum - timenum % 3600) / 3600;
	--print ("Hours "..hours)
	timenum = timenum - hours * 3600
	local minutes = (timenum - timenum % 60) / 60;
	--print ("Minutes "..minutes)
	timenum = timenum - minutes * 60
	local seconds = timenum
	--print ("Minutes "..seconds)
	
	if hours > 0 then
		timestr = timestr..string.format("%d", hours)..":"
	end

	local s; 
	s = tostring(minutes)
	if #s < 2 then
		s = "0"..s
	end
	timestr = timestr ..s..":"

	s = tostring(seconds)
	if #s < 2 then
		s = "0"..s
	end
	timestr = timestr .. s
	
	return timestr;
end
-----------------------------------------------------------------------------------------
-- Make item of specified preset, module and class
-----------------------------------------------------------------------------------------
function CF_MakeItem(item, class, module)
	if MovableMan:GetMOIDCount() < CF_MOIDLimit then
		if class == nil then
			class = "HDFirearm"
		end
		
		if class == "HeldDevice" then
			return CreateHeldDevice(item, module)
		elseif class == "HDFirearm" then
			return CreateHDFirearm(item, module)
		elseif class == "TDExplosive" then
			return CreateTDExplosive(item, module)
		elseif class == "ThrownDevice" then
			return CreateThrownDevice(item, module)
		end
	end
	
	return nil;
end
-----------------------------------------------------------------------------------------
-- Make actor of specified preset, module and class
-----------------------------------------------------------------------------------------
function CF_MakeActor(item, class, module)
	if MovableMan:GetMOIDCount() < CF_MOIDLimit then
		if class == nil then
			class = "AHuman"
		end
		
		if class == "AHuman" then
			return CreateAHuman(item, module)
		elseif class == "ACrab" then
			return CreateACrab(item, module)
		elseif class == "Actor" then
			return CreateActor(item, module)
		elseif class == "ACDropShip" then
			return CreateACDropShip(item, module)
		elseif class == "ACRocket" then
			return CreateACRocket(item, module)
		end
	end
	
	return nil;
end
-----------------------------------------------------------------------------------------
-- Make actor of specified preset, module and class
-----------------------------------------------------------------------------------------
function CF_MakeActorNoMatterWhat(item, class, module)
	if class == nil then
		class = "AHuman"
	end
	
	if class == "AHuman" then
		return CreateAHuman(item, module)
	elseif class == "ACrab" then
		return CreateACrab(item, module)
	elseif class == "Actor" then
		return CreateActor(item, module)
	elseif class == "ACDropShip" then
		return CreateACDropShip(item, module)
	elseif class == "ACRocket" then
		return CreateACRocket(item, module)
	end
	
	return nil;
end
-----------------------------------------------------------------------------------------
-- Get table with inventory of actor, inventory cleared as a result
-----------------------------------------------------------------------------------------
function CF_GetInventory(actor)
	local inventory = {}
	local classes = {}

	if MovableMan:IsActor(actor) then
		if actor.ClassName == "AHuman" then
			local human = ToAHuman(actor);
			
			if human ~= nil then
				if human.EquippedItem ~= nil then
					inventory[#inventory + 1] = human.EquippedItem.PresetName;
					classes[#classes + 1] = human.EquippedItem.ClassName;
				end
				
				human:UnequipBGArm()
			end
		end
		
		if not actor:IsInventoryEmpty() then
			local enough = false;
			while not enough do
				local weap = nil;
			
				weap = actor:SwapNextInventory(weap, true);
				
				if weap == nil then
					enough = true;
				else
					inventory[#inventory + 1] = weap.PresetName;
					classes[#classes + 1] = weap.ClassName;
				end
			end
		end
	else
	end
	
	return inventory, classes;
end
-----------------------------------------------------------------------------
--
-----------------------------------------------------------------------------




















